/**
  ******************************************************************************
  * @file    tsl_acq_HW_stm32l1xx.h
  * @author  MCD Application Team
  * @version V0.4.0
  * @date    06-June-2012
  * @brief   This file contains external declarations of the tsl_acq_stm32l1xx.c file.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __TSL_ACQ_HW_STM32L1XX_H
#define __TSL_ACQ_HW_STM32L1XX_H

/* Includes ------------------------------------------------------------------*/
#include "stm32l1xx.h"
#include "tsl_conf_stm32l1xx.h"
#include "tsl_types.h"

/* Defines -------------------------------------------------------------------*/

#ifndef CONST
#define CONST const
#endif

// SysTick enable/disable interrupt macros
#define enableInterrupts()  {SysTick->CTRL |= SysTick_CTRL_TICKINT_Msk;}
#define disableInterrupts() {SysTick->CTRL &= ~SysTick_CTRL_TICKINT_Msk;}

/** Groups list
  */
enum
{
  GR1,
  GR2,
  GR3,
  GR4,
  GR5,
  GR6,
  GR7,
  GR8,
  GR9,
  GR10,
  GR11
};

/** GPIOs list
  */
enum
{
  PA0 = 0x00, /**<  */
  PA1,
  PA2,
  PA3,
  PA6 = 0x06, /**<  */
  PA7,
  PA8,
  PA9,
  PA10,
  PA11,
  PA13,
  PA14,
  PA15,
  PB0 = 0x10, /**<  */
  PB1,
  PB2,
  PB4 = 0x14, /**<  */
  PB5,
  PB6,
  PB7,
  PB12 = 0x1C, /**<  */
  PB13,
  PB14,
  PB15,
  PC0 = 0x20, /**<  */
  PC1,
  PC2,
  PC3,
  PC4,
  PC5,
  PC6,
  PC7,
  PC8,
  PC9,
  PC10,
  PE7 = 0x47, /**<  */
  PE8,
  PE9,
  PE10,
  PF6 = 0x56, /**<  */
  PF7,
  PF8,
  PF9,
  PF10,
  PF11,
  PF12,
  PF13,
  PF14,
  PF15,
  PG0 = 0x60, /**<  */
  PG1,
  PG2,
  PG3,
  PG4
};

/* Exported types ------------------------------------------------------------*/

typedef struct
{
  // RI
  __IO uint32_t ICR;
  __IO uint32_t ASCR1;
  __IO uint32_t ASCR2;
  __IO uint32_t HYSCR1;
  __IO uint32_t HYSCR2;
  __IO uint32_t HYSCR3;
  __IO uint32_t HYSCR4;
  // CP
  __IO uint32_t ASMR1;
  __IO uint32_t CMR1;
  __IO uint32_t CICR1;
  __IO uint32_t ASMR2;
  __IO uint32_t CMR2;
  __IO uint32_t CICR2;
  __IO uint32_t ASMR3;
  __IO uint32_t CMR3;
  __IO uint32_t CICR3;
  __IO uint32_t ASMR4;
  __IO uint32_t CMR4;
  __IO uint32_t CICR4;
  __IO uint32_t ASMR5;
  __IO uint32_t CMR5;
  __IO uint32_t CICR5;
} CPRI_TypeDef;

// For all devices/acquisitions

typedef uint16_t  TSL_tMeas_T; /**< Measurement */
typedef uint16_t  TSL_tRef_T; /**< Reference */
typedef int16_t   TSL_tDelta_T; /**< Delta */

typedef uint8_t   TSL_tIndexSrc_T; /**< Channel source index */
typedef uint16_t  TSL_tIndexDest_T; /**< Channel destination index */

typedef uint8_t   TSL_tRefRest_T; /**< Reference Rest (ECS) */
typedef uint16_t  TSL_tKCoeff_T; /**< K coefficient (ECS) */

typedef uint8_t   TSL_tIndex_T; /**< Generic index */
typedef uint16_t  TSL_tNb_T; /**< Generic number */
typedef uint8_t   TSL_tCounter_T; /**< Generic counter used for debounce */

typedef uint8_t   TSL_tThreshold_T; /**< Delta threshold */

typedef int16_t   TSL_tsignPosition_T; /**< Linear and Rotary sensors position */
typedef uint8_t   TSL_tPosition_T; /**< Linear and Rotary sensors position */

typedef uint16_t  TSL_tTick_ms_T; /**< Time in ms */
typedef uint8_t   TSL_tTick_sec_T; /**< Time in sec */

//------------------------------------------------------------------------------
// Channel
//------------------------------------------------------------------------------

typedef uint8_t TSL_Conf_t;

/** Channel destination index
  */
typedef struct
{
  TSL_tIndex_T  IdxDest; /**< Index in the Channel data array */
}
TSL_ChannelDest_T;

/** Channel Source and Configuration
  */
typedef struct
{
  TSL_tIndex_T  IdxSrc;  /**< Index of source value */
  // For stm32l1x TSC acquisition only
  TSL_Conf_t          t_sample;  /**< Indicates which GPIO.n is used for the sample */
  TSL_Conf_t          t_channel; /**< Indicates which GPIO.n is used for the channel */
}
TSL_ChannelSrc_T;

/** Channel flags
  */
typedef struct
{
TSL_DataReady_enum_T DataReady : 1; /**< To identify a new measurement */
TSL_AcqStatus_enum_T AcqStatus : 2; /**< Acquisition status */
TSL_ObjStatus_enum_T ObjStatus : 2; /**< Object status */
}
TSL_ChannelFlags_T;

/** Channel Data
  */
typedef struct
{
  TSL_ChannelFlags_T   Flags;     /**< Flags */
  TSL_tRef_T           Ref;       /**< Reference */
  TSL_tRefRest_T       RefRest;   /**< Reference rest for ECS */
  TSL_tDelta_T         Delta;     /**< Delta */
#if TSLPRM_USE_MEAS > 0
  TSL_tMeas_T          Meas;      /**< Hold the last acquisition measure */
#endif
}
TSL_ChannelData_T;

//------------------------------------------------------------------------------
// Bank
//------------------------------------------------------------------------------

/** Bank
  */
typedef struct
{
  // Common to all acquisitions
  CONST TSL_ChannelSrc_T  *p_chSrc;     /**< Pointer to the Channel Source and Configuration */
  CONST TSL_ChannelDest_T *p_chDest;    /**< Pointer to the Channel Destination */
  TSL_ChannelData_T       *p_chData;    /**< Pointer to the Channel Data */
  TSL_tNb_T               NbChannels;   /**< Number of channels in the bank */
  // For stm32l1x TSC acquisition only
  TSL_Conf_t              shield_sample;  /**< Indicates which GPIO.n is used for the shield sample */
  TSL_Conf_t              shield_channel; /**< Indicates which GPIO.n is used for the shield channel */
}
TSL_Bank_T;

/* Exported variables --------------------------------------------------------*/

/* Exported macros -----------------------------------------------------------*/

#define CPRI_BASE  (APB1PERIPH_BASE + 0x7C04)
#define CPRI       ((CPRI_TypeDef *) CPRI_BASE)

/* Exported functions ------------------------------------------------------- */

TSL_Status_enum_T TSL_acq_Init(void);
TSL_Status_enum_T TSL_acq_BankConfig(TSL_tIndex_T idx_bk);
void TSL_acq_BankStartAcq(void);
TSL_Status_enum_T TSL_acq_BankWaitEOC(void);
void TSL_acq_ProcessIT(void);
TSL_tMeas_T TSL_acq_GetMeas(TSL_tIndex_T index);
TSL_AcqStatus_enum_T TSL_acq_CheckNoise(void);
TSL_Bool_enum_T TSL_acq_UseFilter(TSL_ChannelData_T *pCh);
TSL_tDelta_T TSL_acq_ComputeDelta(TSL_tRef_T ref, TSL_tMeas_T meas);
TSL_tMeas_T TSL_acq_ComputeMeas(TSL_tRef_T ref, TSL_tDelta_T delta);
TSL_Bool_enum_T TSL_acq_TestReferenceOutOfRange(TSL_ChannelData_T *pCh);
TSL_Bool_enum_T TSL_acq_TestFirstReferenceIsValid(TSL_ChannelData_T *pCh, TSL_tMeas_T new_meas);

#endif // __TSL_ACQ_HW_STM32L1XX_H

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
