/**
  ******************************************************************************
  * @file    tsl_time_stm8tl5x.c
  * @author  MCD Application Team
  * @version V0.4.0
  * @date    06-June-2012
  * @brief   This file contains all functions to manage the timing with STM8TL5x products.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "tsl_time_stm8tl5x.h"
#include "tsl_time.h"
#include "stm8tl5x_it.h"

/* Private typedefs ----------------------------------------------------------*/
/* Private defines -----------------------------------------------------------*/
/* Private macros ------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private functions prototype -----------------------------------------------*/

/**
  * @brief  Initialization of the timing module to generate one interrupt every 0.5ms
  * @param  None
  * @retval None
  */
void TSL_tim_Init(void)
{
  CLK->PCKENR1 |= CLK_PCKENR1_TIM4; // The peripheral clock are not enable by default

  TIM4->SR1 = 0; // Clear overflow flag

  if (CLK->CKDIVR == 0x00) // Max CPU freq = 16 MHz
  {
    TIM4->PSCR = 6; // Prescaler to divide Fcpu by 64: 4 us clock.
  }

  TIM4->ARR = 124;  // 125 * 4 us = 500 us.
  TIM4->IER = 0x01; // Enable interrupt
  TIM4->CR1 = 0x01; // Start timer
}


/**
  * @brief  Interrupt handler for TIM4 dedicated to ECS
  * @param  None
  * @retval None
  */
#if defined(_COSMIC_)
// 'svlreg option' is added to force the saving of the virtual long register
@svlreg INTERRUPT_HANDLER(TSL_Timer_ISR, 25)
#else
INTERRUPT_HANDLER(TSL_Timer_ISR, 25)
#endif
{
  TIM4->SR1 &= (uint8_t)(~TIM4_SR1_UIF);
  TSL_tim_ProcessIT();
}

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
