/**
  ******************************************************************************
  * @file    STM32F0518_Ex02_Linear_EVAL\src\main.c 
  * @author  MCD Application Team
  * @version V0.1.0
  * @date    15-Juin-2012
  * @brief   Basic example on how to use the TouchSensing Driver on
  *          STM32F0xx devices with STM320518-EVAL board.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */ 

/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include "stm32f0xx.h"
#include "tsl_user_conf.h"
#include "stm320518_eval_lcd.h"
#include "serial.h"

// Select to use or not the LCD
//#define USE_LCD 1 // 0=No, 1=Yes

/* Private typedefs ----------------------------------------------------------*/

/* Private defines -----------------------------------------------------------*/

// LEDs definition on STM320518-EVAL board
// PC10 = pin 51 = LED1
#define LED1_TOGGLE {GPIOC->ODR ^= 0x0400;}
#define LED1_OFF    {GPIOC->BSRR = (uint32_t)((uint32_t)1 << PIN_10);}
#define LED1_ON     {GPIOC->BSRR = (uint32_t)((uint32_t)1 << (PIN_10 + 16));}
// PC11 = pin 52 = LED2
#define LED2_TOGGLE {GPIOC->ODR ^= 0x0800;}
#define LED2_OFF    {GPIOC->BSRR = (uint32_t)((uint32_t)1 << PIN_11);}
#define LED2_ON     {GPIOC->BSRR = (uint32_t)((uint32_t)1 << (PIN_11 + 16));}
// PC12 = pin 53 = LED3
#define LED3_TOGGLE {GPIOC->ODR ^= 0x1000;}
#define LED3_OFF    {GPIOC->BSRR = (uint32_t)((uint32_t)1 << PIN_12);}
#define LED3_ON     {GPIOC->BSRR = (uint32_t)((uint32_t)1 << (PIN_12 + 16));}
// PD2 = pin 54 = LED4
#define LED4_TOGGLE {GPIOD->ODR ^= 0x0004;}
#define LED4_OFF    {GPIOD->BSRR = (uint32_t)((uint32_t)1 << PIN_02);}
#define LED4_ON     {GPIOD->BSRR = (uint32_t)((uint32_t)1 << (PIN_02 + 16));}

/* Private macros ------------------------------------------------------------*/

#define TEST_TKEY(NB) (MyLinRots[(NB)].p_Data->StateId == TSL_STATEID_DETECT)

/* Private variables ---------------------------------------------------------*/

TSL_tTick_ms_T ECS_last_tick; // Hold the last time value for ECS
uint32_t process_sensor;

/* Private functions prototype -----------------------------------------------*/

void TSL_Action(void);
void ProcessSensors(void);

#if USE_LCD > 0
void LcdDisplayStatus(void);
uint8_t LcdMakeLine(uint8_t *output, uint8_t *name, uint8_t *state, uint8_t *data);
uint8_t *LcdState2String(TSL_StateId_enum_T State);
void LcdValue2String(uint8_t *output, int16_t input);
#endif


void UartDisplayStatus(void);
uint8_t* UARTState2String(TSL_StateId_enum_T state);

/**
  * @brief  Main routine.
  * @param  None
  * @retval None
  */
int main(void)
{
  int LED = 0;
  //============================================================================
  // Init system clock
  //============================================================================
  // At this stage the microcontroller clock setting is already configured, 
  // this is done through SystemInit() function which is called from startup
  // file (startup_stm32f0xx.s) before to branch to application main.
  // To reconfigure the default setting of SystemInit() function, refer to
  // system_stm32f0xx.c file

  //============================================================================
  // Init Standard GPIOs
  //============================================================================  
  
  // Enable GPIO clocks
  RCC->AHBENR |= (RCC_AHBENR_GPIOAEN | RCC_AHBENR_GPIOBEN | RCC_AHBENR_GPIOCEN | RCC_AHBENR_GPIODEN);
  
  // LEDs settings
  // Set PC10 in Output Push-Pull 50MHz max (LED1)
  GPIOC->MODER   |= (uint32_t)(MODE_OUT << (PIN_10 * 2)); // General purpose Output mode
  GPIOC->OSPEEDR |= (uint32_t)(SPEED_50 << (PIN_10 * 2)); // 50MHz max speed
  GPIOC->ODR     |= (uint32_t)(1 << PIN_10);  
  // Set PC11 in Output Push-Pull 50MHz max (LED2)
  GPIOC->MODER   |= (uint32_t)(MODE_OUT << (PIN_11 * 2)); // General purpose Output mode
  GPIOC->OSPEEDR |= (uint32_t)(SPEED_50 << (PIN_11 * 2)); // 50MHz max speed
  GPIOC->ODR     |= (uint32_t)(1 << PIN_11);
  // Set PC12 in Output Push-Pull 50MHz max (LED3)
  GPIOC->MODER   |= (uint32_t)(MODE_OUT << (PIN_12 * 2)); // General purpose Output mode
  GPIOC->OSPEEDR |= (uint32_t)(SPEED_50 << (PIN_12 * 2)); // 50MHz max speed
  GPIOC->ODR     |= (uint32_t)(1 << PIN_12);
  // Set PD2 in Output Push-Pull 50MHz max (LED4)
  GPIOD->MODER   |= (uint32_t)(MODE_OUT << (PIN_02 * 2)); // General purpose Output mode
  GPIOD->OSPEEDR |= (uint32_t)(SPEED_50 << (PIN_02 * 2)); // 50MHz max speed
  GPIOD->ODR     |= (uint32_t)(1 << PIN_02);
  

  LED_Init();                                /* LED Initialization            */
  LED_On(0);

  SER_Init();
  printf("USART Initialized\r\n");
  
  // Set PA8 in Alternate function Output Push-Pull 50MHz max (MCO)
  //GPIOA->MODER   |= (MODE_AF  << (PIN_08 * 2)); // Alternate-Function mode
  //GPIOA->OSPEEDR |= (SPEED_50 << (PIN_08 * 2)); // 50MHz max speed
  // Output SYSCLK on MCO pin (PA8)
  //RCC->CFGR |= RCC_CFGR_MCO_SYSCLK;
  
  //============================================================================
  // Init TouchSense GPIOs
  //============================================================================  
  
  Init_TS_GPIOs();
  printf("Init TouchSense GPIOs\r\n");
  
  //============================================================================
  // Init STMTouch driver
  //============================================================================  

  TSL_obj_GroupInit(&MyObjGroup); // Init Objects
  printf("Init Objects\r\n");

  TSL_Init(MyBanks); // Init timing and acquisition modules
  printf("Init timing and acquisition modules\r\n");
  LED_Off(0);
  LED_Off(1);

  //============================================================================
  // Init LCD
  //============================================================================  

#if USE_LCD > 0  
  STM320518_LCD_Init();
  
  LCD_Clear(LCD_COLOR_BLUE);  
  LCD_SetBackColor(LCD_COLOR_WHITE);
  LCD_SetTextColor(LCD_COLOR_BLUE);
  LCD_DisplayStringLine(LCD_LINE_1, "      STM32F051     ");
  LCD_DisplayStringLine(LCD_LINE_2, "      Example 2     ");
  
  LCD_SetBackColor(LCD_COLOR_BLUE);  
  LCD_SetTextColor(LCD_COLOR_WHITE);
#endif
  
	printf("MainLoop\r\n");
  //============================================================================
  // Main loop
  //============================================================================
  for (;;)
  {

    // Execute STMTouch Driver state machine
    TSL_Action();
    
    // Execute Application
    ProcessSensors();
	LED_Out(LED); LED++; LED&=3;

  }
 
}


/**
  * @brief  Execute STMTouch Driver main State machine
  * @param  None
  * @retval None
  */
void TSL_Action(void)
{
  uint8_t idx_bank;

  // Acquire all Banks
  for (idx_bank = 0; idx_bank < TSLPRM_TOTAL_BANKS; idx_bank++)
  {  
    // Configure Bank
    TSL_acq_BankConfig(idx_bank);
 
    // Start Bank acquisition
    TSL_acq_BankStartAcq();

    // Wait Bank End of Acquisition
    while (TSL_acq_BankWaitEOC() == TSL_STATUS_BUSY) {}

    // Get Bank Result
    TSL_acq_BankGetResult(idx_bank, 0, 0);
  }

  // Process Objects
  TSL_obj_GroupProcess(&MyObjGroup);
  
  // DxS processing (if TSLPRM_USE_DXS option is set)
  TSL_dxs_FirstObj(&MyObjGroup);

  // ECS every 100ms
  if (TSL_tim_CheckDelay_ms(100, &ECS_last_tick) == TSL_STATUS_OK)
  {
    if (TSL_ecs_Process(&MyObjGroup) == TSL_STATUS_OK)
    {
      LED4_TOGGLE;
      process_sensor = 0;
    }
    else
    {
      process_sensor = 1;
    }
  }
  
}


/**
  * @brief  Manage the activity on sensors when touched/released (example)
  * @param  None
  * @retval None
  */
void ProcessSensors(void)
{

  if (!process_sensor) return;

  LED1_OFF;
  LED2_OFF;
  LED3_OFF;
  LED4_OFF;

  if (MyLinRots[0].p_Data->RawPosition >= 5)
  {
    LED1_ON;
  }

  if (MyLinRots[0].p_Data->RawPosition >= 64)
  {
    LED2_ON;
  }

  if (MyLinRots[0].p_Data->RawPosition >= 128)
  {
    LED3_ON;
  }

  if (MyLinRots[0].p_Data->RawPosition >= 192)
  {
    LED4_ON;
  }

  UartDisplayStatus();

#if USE_LCD > 0
  LcdDisplayStatus();
#endif

}

void UartDisplayStatus(void)
{
  uint8_t TheLine[20];
  uint8_t strPosition[5]; // sign + 3 digits + empty char
  uint8_t strDelta[5]; // sign + 3 digits + empty char
  
   if ((MyLinRots[0].p_Data->Change) || (MyLinRots[0].p_Data->PosChange))
   {
     
     MyLinRots[0].p_Data->PosChange = TSL_STATE_NOT_CHANGED;

	 printf("LTS %3d %s ",MyLinRots[0].p_Data->Position,UARTState2String(MyLinRots[0].p_Data->StateId));
	 printf("CH0: %2d ",MyLinRots[0].p_ChD[0].Delta);
	 printf("CH1: %2d ",MyLinRots[0].p_ChD[1].Delta);
	 printf("CH2: %2d ",MyLinRots[0].p_ChD[2].Delta);
	 printf("\r\n");
   }
  
}

/**
  * @brief Converts the sensor state ID in a string
  * @param[in] state Sensor state ID
  * @retval String Sensor state string
  */
uint8_t* UARTState2String(TSL_StateId_enum_T state)
{
  switch (state)
  {
    case TSL_STATEID_DEB_CALIB:
        return "DB_CALIB";
    case TSL_STATEID_CALIB:
        return "CALIB   ";
    case TSL_STATEID_RELEASE:
        return "RELEASE ";
    case TSL_STATEID_DEB_RELEASE_PROX:	
    case TSL_STATEID_DEB_RELEASE_DETECT:
    case TSL_STATEID_DEB_RELEASE_TOUCH:
        return "DB_REL  ";
    case TSL_STATEID_PROX:
        LCD_SetColors(LCD_COLOR_YELLOW, LCD_COLOR_BLUE);
        return "PROX    ";
    case TSL_STATEID_DEB_PROX:
    case TSL_STATEID_DEB_PROX_DETECT:
    case TSL_STATEID_DEB_PROX_TOUCH:
        return "DB_PROX ";
    case TSL_STATEID_DETECT:
        LCD_SetColors(LCD_COLOR_GREEN, LCD_COLOR_BLUE);
        return "DETECT  ";
    case TSL_STATEID_DEB_DETECT:
        return "DB_DET  ";
    case TSL_STATEID_TOUCH:
        LCD_SetColors(LCD_COLOR_RED, LCD_COLOR_BLUE);
        return "TOUCH   ";
    case TSL_STATEID_ERROR:
        return "ERROR   ";
    case TSL_STATEID_DEB_ERROR_CALIB:
    case TSL_STATEID_DEB_ERROR_RELEASE:
    case TSL_STATEID_DEB_ERROR_PROX:
    case TSL_STATEID_DEB_ERROR_DETECT:
    case TSL_STATEID_DEB_ERROR_TOUCH:
        return "DB_ERR  ";
    case TSL_STATEID_OFF:
        return "OFF     ";
    default:
        return "UNKNOWN ";
  }
}


#if USE_LCD > 0

/**
  * @brief  Displays the state and position of linear sensor
  * @param  None
  * @retval None
  */
void LcdDisplayStatus(void)
{
  uint8_t TheLine[20];
  uint8_t strPosition[5]; // sign + 3 digits + empty char
  uint8_t strDelta[5]; // sign + 3 digits + empty char
  
   if ((MyLinRots[0].p_Data->Change) || (MyLinRots[0].p_Data->PosChange))
   {
     
     MyLinRots[0].p_Data->PosChange = TSL_STATE_NOT_CHANGED;
     
     LcdValue2String(strPosition, MyLinRots[0].p_Data->Position); // Convert position in string
     LcdMakeLine(TheLine, "LTS ", LcdState2String(MyLinRots[0].p_Data->StateId), strPosition);
     LCD_DisplayStringLine(LCD_LINE_5, TheLine);
     
     LCD_SetColors(LCD_COLOR_WHITE, LCD_COLOR_BLUE);

     LcdValue2String(strDelta, MyLinRots[0].p_ChD[0].Delta); // Convert delta in string
     LcdMakeLine(TheLine, "CH0 ", "DELTA   ", strDelta);
     LCD_DisplayStringLine(LCD_LINE_6, TheLine);

     LcdValue2String(strDelta, MyLinRots[0].p_ChD[1].Delta); // Convert delta in string
     LcdMakeLine(TheLine, "CH1 ", "DELTA   ", strDelta);
     LCD_DisplayStringLine(LCD_LINE_7, TheLine);

     LcdValue2String(strDelta, MyLinRots[0].p_ChD[2].Delta); // Convert delta in string
     LcdMakeLine(TheLine, "CH2 ", "DELTA   ", strDelta);
     LCD_DisplayStringLine(LCD_LINE_8, TheLine);
     
   }
  
}



/**
  * @brief Converts the sensor state ID in a string
  * @param[in] state Sensor state ID
  * @retval String Sensor state string
  */
uint8_t* LcdState2String(TSL_StateId_enum_T state)
{
  LCD_SetColors(LCD_COLOR_WHITE, LCD_COLOR_BLUE);
  switch (state)
  {
    case TSL_STATEID_DEB_CALIB:
        return "DB_CALIB";
    case TSL_STATEID_CALIB:
        return "CALIB   ";
    case TSL_STATEID_RELEASE:
        return "RELEASE ";
    case TSL_STATEID_DEB_RELEASE_PROX:	
    case TSL_STATEID_DEB_RELEASE_DETECT:
    case TSL_STATEID_DEB_RELEASE_TOUCH:
        return "DB_REL  ";
    case TSL_STATEID_PROX:
        LCD_SetColors(LCD_COLOR_YELLOW, LCD_COLOR_BLUE);
        return "PROX    ";
    case TSL_STATEID_DEB_PROX:
    case TSL_STATEID_DEB_PROX_DETECT:
    case TSL_STATEID_DEB_PROX_TOUCH:
        return "DB_PROX ";
    case TSL_STATEID_DETECT:
        LCD_SetColors(LCD_COLOR_GREEN, LCD_COLOR_BLUE);
        return "DETECT  ";
    case TSL_STATEID_DEB_DETECT:
        return "DB_DET  ";
    case TSL_STATEID_TOUCH:
        LCD_SetColors(LCD_COLOR_RED, LCD_COLOR_BLUE);
        return "TOUCH   ";
    case TSL_STATEID_ERROR:
        return "ERROR   ";
    case TSL_STATEID_DEB_ERROR_CALIB:
    case TSL_STATEID_DEB_ERROR_RELEASE:
    case TSL_STATEID_DEB_ERROR_PROX:
    case TSL_STATEID_DEB_ERROR_DETECT:
    case TSL_STATEID_DEB_ERROR_TOUCH:
        return "DB_ERR  ";
    case TSL_STATEID_OFF:
        return "OFF     ";
    default:
        return "UNKNOWN ";
  }
}


/**
  * @brief Converts a value in string
  * @param[out] output String
  * @param[in]  input  Value
  * @retval None
  */
void LcdValue2String(uint8_t *output, int16_t input)
{
  uint8_t *temp;
  temp = output;

  if ((input > 999) || (input < -999))
  {
    *temp = 0;
    return;
  }
  
  if (input < 0)
  {
    *temp = '-';
    input = -input;
  }
  else
  {
    *temp = ' ';
  }

  temp++;
  
  if ((input > 99) && (input < 1000))
    *temp = 48 + (input / 100);
  else
    *temp = ' ';
    
  temp++;
  
  if (input > 9)
    *temp = 48 + ((input % 100) / 10);
  else
    *temp = ' ';
    
  temp++;
  
  *temp = 48 + (input % 10);
  
  temp++;

  *temp = 0;
}


/**
  * @brief Create the string to be displayed
  * @param[out] output Output string
  * @param[in]  name   Sensor name (4 characters)
  * @param[in]  state  Sensor state (8 characters)
  * @param[in]  data   Sensor data
  * @retval uint8_t Number of characters in the output string
  */
uint8_t LcdMakeLine(uint8_t *output, uint8_t *name, uint8_t *state, uint8_t *data)
{
  uint8_t index = 0;
  uint8_t index2;
  
  // Sensor name: 4 characters
  for (index2 = 0; index2 < 4; index2++)
  {
    output[index++] = name[index2];
  }
  output[index++] = ' ';

  // Sensor state: 8 characters  
  for (index2 = 0; index2 < 8; index2++)
  {
    output[index++] = state[index2];
  }
  output[index++] = ' ';
  
  // Sensor data
  for (index2 = 0; data[index2] != 0; index2++)
  {
    output[index++] = data[index2];
  }
  
  output[index++] = 0;
  
  return index;
}

#endif // USE_LCD


/**
  * @brief  Executed when a sensor is in Error state
  * @param  None
  * @retval None
  */
void MyLinRots_ErrorStateProcess(void)
{
  // Add here your own processing when a sensor is in Error state
  TSL_linrot_SetStateOff();
}


/**
  * @brief  Executed when a sensor is in Off state
  * @param  None
  * @retval None
  */
void MyLinRots_OffStateProcess(void)
{
  // Add here your own processing when a sensor is in Off state
  LED1_OFF;
  LED2_OFF;
  LED3_ON;
  LED4_OFF;
  process_sensor = 0;  
}


//-------------------
// CallBack functions
//-------------------

/**
  * @brief  Executed at each timer interruption (option must be enabled)
  * @param  None
  * @retval None
  */
void TSL_CallBack_TimerTick(void)
{
  //LED1_TOGGLE;
}


#ifdef USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  for (;;)
  {
  }
}
#endif

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
